import { View } from '@tarojs/components'
import Taro, { useDidShow } from '@tarojs/taro'
import type { FC } from 'react'
import { memo, useCallback, useEffect, useLayoutEffect, useMemo, useRef, useState, useTransition } from 'react'
import classnames from 'classnames'
import styles from './index.module.scss'
import ItemList, { BackEndSaleModeListField, BackEndSaleModeListFieldMap } from './components/shoppingCartItem/index'
import BottomSettleBar from './components/bottomSettleBar'
import BottomEditBar from './components/bottomEditBar'
import { ShoppingProvider } from './components/shoppingCart/index'
import type { Goods } from './context'
import { ShoppingDispatchType, useShoppingDispatch, useShoppingState } from './context'
import { events } from './context/shoppingStore'
import { dataLoadingStatus, debounce, getFilterData } from '@/common/util'
import { ShoppingCartDeleteApi, ShoppingCartListApi } from '@/api/index'
import { formatMeterDiv, formatPriceDiv } from '@/common/format'
import InfiniteScroll from '@/components/infiniteScroll'
import IconText from '@/components/iconText'
import Search from '@/components/search'
import { alert, goLink, isEmptyObject } from '@/common/common'
import { EnumSaleMode } from '@/common/Enumerate'
import { useCheckTabBarSelectStatus } from '@/use/useCommon'
import { TabBarIndex } from '@/reducers/tabBar'

interface SearchOptions {
  short_name_or_phone?: string
  page: number
  size: number
}

const ShoppingCartContainer: FC = () => {
  const isFirst = useRef(true)

  console.log('rerender component ShoppingCartContainer')

  const { isMultipleSelection, isManageStatus, selectedAmount, currentCheckedPurchaserId } = useShoppingState()
  console.log('rerender selectedAmount', selectedAmount)

  const dispatch = useShoppingDispatch()

  // 管理
  const onStartToManage = () => {
    dispatch({ type: ShoppingDispatchType.UPDATE_MANAGE_STATUS, data: !isManageStatus })
  }

  const listHeightRef = useRef('auto')

  const { fetchData, state } = ShoppingCartListApi()

  const [searchOptions, setSearchOptions] = useState<SearchOptions>({
    page: 1,
    size: 10,
  })

  const pageNum = useRef({ size: searchOptions.size, page: searchOptions.page })
  const getData = async() => {
    const res = await fetchData(getFilterData(searchOptions))
    // clearMultiSelection()
    if (!res.success) {
      alert.error('请求失败')
      setRefreshStatus(false)
    }
    setRefreshStatus(false)
  }

  useDidShow(async() => {
    // 第二次进入该页面时触发
    if (!isFirst.current) {
      console.log('重新刷新')
      // 重新刷新
      getData()
    }
  })

  useEffect(() => {
    console.log('useEffect fetchData', getFilterData(searchOptions))
    if (!isFirst.current) {
      getData()
    }
  }, [searchOptions])

  // 输入了搜索关键字
  const getSearchData = useCallback(debounce((e) => {
    console.log('getSearchData===>', e)
    setSearchOptions({ short_name_or_phone: e, size: 10, page: 1 })
  }, 300), [])

  const [shoppingCartData, setShoppingCartData] = useState<{
    list: ShoppingCartData[]
    total: number
  }>({ list: [], total: 0 })

  // 数据加载状态
  const statusMore = useMemo(() => {
    console.log('shoppingCartData==>', shoppingCartData, state)
    const status = dataLoadingStatus({
      list: shoppingCartData.list,
      total: shoppingCartData.total,
      status: state.loading!,
    })
    console.log('status==>', status)
    return status
  }, [shoppingCartData, state])

  // useLayoutEffect 执行在DOM更新之后，浏览器绘制之前 如果放在 useEffect 里面会产生多一次不必要的回流和重绘，可能会引起视图闪现
  useLayoutEffect(() => {
    (async() => {
      console.log('useLayoutEffect')
      await fetchData(getFilterData(searchOptions))
      isFirst.current = false
    })()
  }, [])

  useEffect(() => {
    if (state.success) {
      console.log('fetch success', state)
      setShoppingCartData({ list: state.data.list, total: state.data.total })
    }
  }, [state])

  // 结算
  const handleSettleAccount = debounce(() => {
    events.trigger('SettleAccount', currentCheckedPurchaserId)
  }, 400)

  // const { fetchData: deleteApi } = ShoppingCartDeleteApi()

  const onDeleteItem = useCallback(async() => {
    const deleted = await fetchData(searchOptions)
    console.log('onDeleteItem')
    events.trigger('handleOtherHighLight', currentCheckedPurchaserId, deleted.data.list.find(item => item.purchaser_id === currentCheckedPurchaserId))
  }, [])

  // 批量某个客户的删除商品
  const handleDelete = () => {
    console.log('handleDelete')
    events.trigger('DeleteSpecificItem', currentCheckedPurchaserId)
  }
  // 全选
  const handleSelectAllCheckbox = (isSelectAll: boolean) => {
    if (!currentCheckedPurchaserId) { return alert.error('请先选择客户') }

    if (isSelectAll) {
      dispatch({ type: ShoppingDispatchType.UPDATE_MULTIPLE_SELECTION_STATUS, data: isSelectAll })
    }
    else {
      events.trigger('CleanMultiSelection', currentCheckedPurchaserId)
    }
  }

  // 加载刷新数据
  const [refreshStatus, setRefreshStatus] = useState(false)

  // 下拉刷新
  const handleRefresh = async() => {
    pageNum.current.page = 1
    setRefreshStatus(true)
    setSearchOptions(e => ({ ...e, size: 10 }))
    // const res = await fetchData(getFilterData(searchOptions))
    // if (res.success) {
    //   // clearMultiSelection()
    //   setRefreshStatus(false)
    //   alert.success('刷新成功')
    // }
    // else {
    //   alert.error('刷新失败')
    //   setRefreshStatus(false)
    // }
  }
  // const [isPending, startTransition] = useTransition()

  useCheckTabBarSelectStatus(TabBarIndex.SHOPPING)

  const getScrolltolower = useCallback(() => {
    if (shoppingCartData.list.length < shoppingCartData.total) {
      pageNum.current.page++
      const size = pageNum.current.size * pageNum.current.page
      setSearchOptions(e => ({ ...e, size }))
    }
  }, [shoppingCartData])

  return (
    <View className={classnames('flex-col', styles.shopping)} id="shoppingContainer">
      <View className={styles['shopping--topBar']} id="topBar">
        <Search placeholder="请输入客户名称" showBtn={false} changeOnSearch={getSearchData}>
          <View className={styles.flexBox} onClick={onStartToManage}>
            {isManageStatus
              ? (
                <IconText svg iconName="icon-guanlidingdan" text="取消" color="#4581ff" customClass={styles['icon--manage--cancel']} />
              )
              : (
                <IconText svg iconName="icon-guanlidingdan" text="管理" />
              )}
          </View>
        </Search>
      </View>
      <View className={classnames('flex-item', 'flex-col', styles['shopping--context'])}>
        <View id="shoppingListContainer" className={classnames(styles.shopping__list__container, 'flex-item')} style={{ height: listHeightRef.current }}>
          <InfiniteScroll selfonScrollToLower={getScrolltolower} safeAreaInsetBottom={false} statusMore={statusMore} refresherEnabled selfOnRefresherRefresh={handleRefresh} refresherTriggered={refreshStatus}>
            {
              !!shoppingCartData?.list?.length
              && shoppingCartData?.list?.map((item, index) => {
                return <ItemList itemData={item} key={index} onDeleteItem={onDeleteItem}></ItemList>
              })
            }
          </InfiniteScroll>
        </View>
        <View className={styles.bottomBar}>
          {isManageStatus
            ? (
              <BottomEditBar
                disabled={currentCheckedPurchaserId < 0}
                isSelectAll={isMultipleSelection}
                onDelete={handleDelete}
                onSelectCheckbox={isAll => handleSelectAllCheckbox(isAll)}
              ></BottomEditBar>
            )
            : (
              <BottomSettleBar onSettleAccount={handleSettleAccount} amount={selectedAmount}></BottomSettleBar>
            )}
        </View>
      </View>
    </View>
  )
}
export const Shopping: FC = () => {
  // 计算总的预估金额

  return (
    <ShoppingProvider>
      <ShoppingCartContainer />
    </ShoppingProvider>
  )
}

export default memo(Shopping)
